#include <stdio.h>
#include <qtXLS.h>

void set_qT_SQLColumn( qT_SQLColumn *tColumn, 
							  char* sName,
							  void *ArrayAddr,
							  qt_K_INT4 iArrayDim, 
							  qt_K_INT4 iArrayType, 
							  qt_K_INT4 iLENArrElem )
{
	// tColumn->Name = sName;	// column name
	strcpy(tColumn->Name, sName);	// column name
	tColumn->ArrayAddr	= (qt_K_LP)ArrayAddr;		// address of array
	tColumn->ArrayDim		= iArrayDim;		// array dimension
	tColumn->ArrayType	= iArrayType;		// type of array
	tColumn->LENArrElem	= iLENArrElem;    // size of an array element (in bytes)
	tColumn->IndArrAddr	= 0;					// reserved, unused (should be 0)

	return;
}

int main(void)
{
// Program demonstrates usage of qtXLS routines to import some data from 
// an EXCEL file.
//
// We are going to read an EXCEL file named "qtXLSDemo4.xls"
// which contains a table named "qtXLSDemoTable" with
// columns:  lfdNr   x   y   Description   Date_Time   

// Arrays for data to be imported.
	const qt_K_INTEGER  NoColumns = 5;
	char *szTextArr;			//	CHARACTER(256), ALLOCATABLE :: szTextArr(:)
	qt_K_INTEGER *lfdNrArr;			// for INTEGER*4 array
	qt_K_R8 *xArr, *yArr;			// for REAL*8 arrays
	struct qT_TIMESTAMP_STRUCT *TSArr;		// for array of date & time structures

// variables to be used by qtXLS routines
	qt_K_HANDLE hDS;
	qt_K_INT4 iError, iRet, iRow, NoRows, ind;
	char *szFileName;
	struct qT_SQLColumn *tColumns[NoColumns];
	char *szTableName;
	char *szCondition;
	char *szOrderBy;	

	// CALL qtSetXLSLicencePath( szPathName )	// change the path to a location where the licence file resides

// open EXCEL file
// ---------------
	szFileName = "qtXLSDemo4.xls";
	hDS = qtXLSOpenEXCELFile( szFileName );		// returns a "data source handle" to be used with other qtXLS routines
   if ( hDS == 0 )
	{
		printf("Error, qtXLSOpenEXCELFile = %d\n", hDS);
		return -1;
	}
	else
		printf("qtXLSOpenEXCELFile opens %s\n", szFileName);
	printf("\n");

	qtXLSSetErrorLevel( 1 );					// continue, if an error occurs (if possible)
	qtXLSSetErrorMessagesDisplay( 1 );		// turn on "error display"

// get row count of table
// ----------------------
	szTableName = "qtXLSDemoTable";			// table name (zero terminated)
	NoRows = qtXLSGetRowCount( hDS, szTableName );

	if ( NoRows <= 0 ) 
	{
		printf("Table qtXLSDemoTable is empty. No rows to read.\n");
		goto M900;		// to the "Exit"
	}

// allocate arrays for result set
	//szTextArr[NoRows] = new char [NoRows][256];
	szTextArr = (char *) calloc(NoRows, 256);
	lfdNrArr = new qt_K_INTEGER [NoRows];
	xArr = new qt_K_R8 [NoRows];
	yArr = new qt_K_R8 [NoRows];
	TSArr = new qT_TIMESTAMP_STRUCT [NoRows];

	if (		szTextArr == NULL 
			|| lfdNrArr == NULL 
			|| xArr == NULL 
			|| yArr == NULL 
			|| TSArr == NULL )
	{
		printf("Could not allocate arrays.n");
		goto M910;		// to the "Exit"
	}

// set up columns "lfdNr   x   y   Description   Date_Time" for import
// -------------------------------------------------------------------
	// create array of structure tColumns
	tColumns[0] = (qT_SQLColumn *) calloc(NoColumns, sizeof(qT_SQLColumn));
	for (ind = 1; ind < NoColumns; ind++)
		tColumns[ind] = tColumns[ind-1] + 1; // + sizeof(qT_SQLColumn);

// 1st column
	set_qT_SQLColumn( tColumns[0], 
							"lfdNr", 
							lfdNrArr, 
							NoRows,
							qt_SQL_C_SLONG, 
							4 );
// and remaining columns
	set_qT_SQLColumn(tColumns[1], "x", xArr, NoRows, qt_SQL_C_DOUBLE, 8);
	set_qT_SQLColumn(tColumns[2], "y", yArr, NoRows, qt_SQL_C_DOUBLE, 8);
   set_qT_SQLColumn(tColumns[3], "Description", szTextArr, NoRows, qt_SQL_C_CHAR, 256);
   set_qT_SQLColumn(tColumns[4], "Date_Time", TSArr, NoRows, qt_SQL_C_TIMESTAMP, 16);

// read rows
// ---------
	// specification of a condition and the sort order (SQL syntax)
	szCondition = "WHERE x > 0.4 AND x <= 0.5";
	szOrderBy = "ORDER BY x DESC";	// sort order: by column "x", descending

	NoRows = qtXLSReadRows( hDS, 
									szTableName, 
									NoColumns, 
									-1, 
									tColumns[0],	
									szCondition, 
									szOrderBy );

	if ( NoRows < 0 )
		printf("An error occurred performing qtXLSReadRows.\n");
	else
	{
		printf("\n");
		printf("Read all rows of table  %s  %s\n", szTableName, szCondition );
		printf("Sort: %s\n", szOrderBy);
		printf("%d rows read:\n", NoRows);
		printf("lfdNr    x    Description                   y      Date_Time\n");
      //6000 FORMAT( I3.3,2X, 2X, F5.2,2X, A24,2X, F9.6,2X, I4, 2(".",I2.2),1X, 3(I2.2,":"), I3.3)
		for (iRow = 0; iRow < NoRows; iRow++)
		{
			printf("%3d", lfdNrArr[iRow]);
			printf("    %5.2f", xArr[iRow]);
			printf("  %24s", szTextArr + (iRow * 256));
			printf("  %9.6f", yArr[iRow]);
			printf("  %4d.%02d.%02d %02d:%02d:%02d:%03d\n", 
				TSArr[iRow].year, 
				TSArr[iRow].month, 
				TSArr[iRow].day, 
				TSArr[iRow].hour, 
				TSArr[iRow].minute, 
				TSArr[iRow].second, 
				TSArr[iRow].fraction 
				);
		}
	}

// Exit
M910:
	delete szTextArr;
	delete lfdNrArr;
	delete xArr;
	delete yArr;
	delete TSArr;
	delete *tColumns;

M900:
	iRet = qtXLSCloseEXCELFile( hDS );
	if ( iRet == 0 )
	{
		printf("Data successfully read from EXCEL file %s\n", szFileName);
		printf("qtXLS closed.\n");
	}
	else
		printf("Error returned from qtXLSCloseEXCELFile = %d\n", iRet);

	printf("\n");
	printf("(C) QT software GmbH, Germany. All rights reserved. 2006.\n");
	printf("    http:\\www.qtsoftware.de    eMail: info@qtsoftware.de\n");
	return 0;
}