PROGRAM qtXLSDemoReadTable
! Program demonstrates usage of qtXLS routines to import some data from 
! an EXCEL file.
!
! We are going to read an EXCEL file named "qtXLSDemo4.xls"
! which contains a table named "qtXLSDemoTable" with
! columns:  lfdNr   x   y   Description   Date_Time   

	USE qtXLS		! qtXLS MODULE contains declarations and INTERFACES
	IMPLICIT NONE	! Names of variables and constants start with "qt".

! Arrays for data to be imported.
	INTEGER, PARAMETER ::  NoColumns = 5
	CHARACTER(256), ALLOCATABLE :: szTextArr(:)
	INTEGER, ALLOCATABLE :: lfdNrArr(:)							! INTEGER*4
	REAL (qt_K_R8), ALLOCATABLE :: xArr(:), yArr(:)			! REAL*8 arrays
	TYPE (qT_TIMESTAMP_STRUCT), ALLOCATABLE :: TSArr(:)	! defines a date & time structure

! variables to be used by qtXLS routines
	INTEGER (qt_K_HANDLE) hDS
	INTEGER (qt_K_INT4) iError, iRet, iLen, iRow, NoRows
	CHARACTER (20) szFileName
	TYPE (qT_SQLColumn) tColumns(NoColumns)
	CHARACTER (qt_I_MaxTableNameLEN) szTableName
	CHARACTER (qt_I_MaxStatementLEN) szCondition
	CHARACTER (qt_I_MaxStatementLEN)	szOrderBy


   ! prior usage of any other qtXLS function either provide the licence 
   ! file ( (L0611-######.lic, supplied with ealier versions of qtXLS)
   ! or call qtSetLicence_qtXLS(). Otherwise qtXLS runs in demo mode only.)
   !
   ! (1) in case of usage of the licence file (L0611-######.lic), you 
   !     might want to set the path such that the licence file can be found
   ! CALL qtSetXLSLicencePath( szPathName )	! change the path to a location where the licence file resides
   !
   ! (2) if you prefer to have the licence linked into your program
   !     call this routine
   CALL qtSetLicence_qtXLS( iError )    ! supplied in source form (file name: qtSetLicence_0611_######.f90)

! open EXCEL file
! ---------------
	szFileName = 'qtXLSDemo4.xls' // CHAR(0)
	hDS = qtXLSOpenEXCELFile( szFileName )		! returns a "data source handle" to be used with other qtXLS routines
	IF ( hDS == 0 ) THEN
		PRINT*, 'Error when trying to open file ', szFileName
		STOP
	ELSE
		PRINT*, 'qtXLSOpenEXCELFile opened the file ', szFileName
	END IF

	CALL qtXLSSetErrorLevel( 1 )					! continue, if an error occurs (if possible)
	CALL qtXLSSetErrorMessagesDisplay( 1 )		! turn on "error display"

! get row count of table
! ----------------------
	szTableName = 'qtXLSDemoTable' // CHAR(0)			! table name (zero terminated)
	NoRows = qtXLSGetRowCount( hDS, szTableName )

	IF ( NoRows == 0 ) THEN
		PRINT*, 'Table qtXLSDemoTable is empty. No rows to read.'
		GOTO 900		! to the "Exit"
	END IF

! allocate arrays for result set
	ALLOCATE(	szTextArr(NoRows),	&
					lfdNrArr(NoRows),		&
					xArr(NoRows),			&
					yArr(NoRows),			&
					TSArr(NoRows),	STAT = iError)
	IF ( iError /= 0 ) THEN
		PRINT*, 'Could not allocate arrays. Runtime error =', iError
		GOTO 910		! to the "Exit"
	END IF

! set up columns "lfdNr   x   y   Description   Date_Time" for import
! -------------------------------------------------------------------
! 1st column
	tColumns(1) % Name			= 'lfdNr'					! column name
	tColumns(1) % ArrayAddr		= LOC(lfdNrArr)			! memory address of array
	tColumns(1) % ArrayDim		= NoRows						! array dimension
	tColumns(1) % ArrayType		= qt_SQL_C_SLONG			! 4 byte (long) INTEGER
	tColumns(1) % LENArrElem	= 4                     ! size of an array element (in bytes)
	tColumns(1) % IndArrAddr	= 0							! reserved, unused (should be 0)
! and remaining columns (using the TYPE constructor function qT_SQLColumn)
	tColumns(2) = qT_SQLColumn('x', LOC(xArr), NoRows, qt_SQL_C_DOUBLE, 8, 0)
	tColumns(3) = qT_SQLColumn('y', LOC(yArr), NoRows, qt_SQL_C_DOUBLE, 8, 0)
   tColumns(4) = qT_SQLColumn('Description', LOC(szTextArr), NoRows, qt_SQL_C_CHAR,  &
                              LEN(szTextArr(1)), 0)
   tColumns(5) = qT_SQLColumn('Date_Time', LOC(TSArr), NoRows, qt_SQL_C_TIMESTAMP, 16, 0)

! read rows
! ---------
	! specification of a condition and the sort order (SQL syntax)
	szCondition = 'WHERE x > 0.4 AND x <= 0.5' // CHAR(0)
	szOrderBy = 'ORDER BY x DESC' // CHAR(0)		! sort order: by column "x", descending
	
	NoRows = qtXLSReadRows( hDS, szTableName, NoColumns, -1, tColumns,	szCondition, szOrderBy )

	IF ( NoRows < 0 ) THEN
		PRINT*, 'An error occurred performing qtXLSReadRows.'
	ELSE
		PRINT*
		PRINT*, 'Read all rows of table  ',                                      &
                  szTableName(1:qtXLSGetszStringLength( szTableName )), '  ',  &
                  szCondition(1:qtXLSGetszStringLength( szCondition ))
		PRINT*, 'Sort: ', szOrderBy(1:qtXLSGetszStringLength( szOrderBy ))
		PRINT*, NoRows, ' rows read:'
		WRITE (*,6030)
		!            12345            1           123546879 123456789 1234
      6030 FORMAT('lfdNr',  2X, 2X,'x',2X, 2X, 'Description             ', 2X, 4X,'y',4X, 2X, 'Date_Time' )
      6000 FORMAT( I3.3,2X, 2X,   F5.2,    2X,  A24,                       2X,   F9.6,    2X, I4, 2('.',I2.2),   &
                   1X, 3(I2.2,':'), I3.3)
		DO iRow = 1, NoRows
			iLen = qtXLSGetszStringLength( szTextArr(iRow) )
         WRITE (*, 6000) lfdNrArr(iRow), xArr(iRow), szTextArr(iRow)(1:iLen), yArr(iRow), TSArr(iRow)
		END DO
	END IF

! Exit
910	CONTINUE
	DEALLOCATE(	szTextArr, lfdNrArr,	xArr,	yArr,	TSArr, STAT = iError)

900	CONTINUE
	iRet = qtXLSCloseEXCELFile( hDS )
	IF ( iRet == 0 ) THEN
		PRINT*, 'Data successfully read from EXCEL file ', szFileName
		PRINT*, 'qtXLS closed.'
	ELSE
		PRINT*, 'Error returned from qtXLSCloseEXCELFile = ', iRet
	END IF

	PRINT*
	PRINT*, '(C) QT software GmbH, Germany. All rights reserved. 2003-2007.'
	PRINT*, '    http://www.qtsoftware.de    eMail: info@qtsoftware.de'
	PRINT*
	PAUSE 'Press Enter/Return to terminate.'
	STOP 'Program terminated.'

END